/*
  Program: Hromdkov Nim
  Autor: Radek Pelnek  
  Ploha ke knize "Jak to vyeit",  www.radekpelanek.cz/?jtv

  Hraje hru Nim, verzi "hromdkov Nim". Uivatelsk rozhran je velmi
  jednoduch a nijak nekontroluje vstupy od uivatele.
  
*/

#include <iostream>
#include <list>
#include <vector>
#include <stdlib.h>
#include <time.h>
#include <math.h>

using namespace std;

vector<int> hromadka;
int n; // pocet hromadek
bool hrac_na_tahu;

bool vsechny_hromadky_prazdne() {
  for (int i=0; i<n; i++)
    if (hromadka[i]>0) return false;
  return true;
}

void nahodny_tah(int& h, int& k) {
  k = 0;
  do {
    h = rand() % n;
    if (hromadka[h] > 0) 
      k = (rand() % hromadka[h]) + 1;
  } while (k == 0);  
}

// pokus se najt optimln tah, pokud nenajde, nech h a k nulov
void optimalni_tah(int& h, int& k) {
  h = k = 0;
  bool bin_kod[10][32], celkova_parita[32]; // pedpokldme max. 10 hromdek a 32 bitov sla
  int i, x;  // i - itertor pes bity, x - itertor pes hromdky
  
  for (i=0; i<32; i++) celkova_parita[i] = 0;

  for (x=0; x<n; x++) {
    int t = hromadka[x];
    for (i=0; i<32; i++) {
      bin_kod[x][i] = t % 2;
      if (bin_kod[x][i]) celkova_parita[i] = ! celkova_parita[i];
      t /= 2;
    }    
  }
  int poz = -1;
  // najdeme nejvyznamj lichou pozici
  for (i=0; i<32; i++)  if (celkova_parita[i]) poz = i;
  if (poz == -1) return; // sud stav, neexistuje vtzn tah

  // vybereme hromdku, kter m na pslun pozici 1
  for (x=0; x<n; x++) if (bin_kod[x][poz]) h = x;

  // douc stav -- tolik sirek, abychom celkovou paritu dorovnali na sudou
  int zadouci = 0;
  for (i=0; i<32; i++) {
    if (celkova_parita[i] ^ bin_kod[h][i]) {
      zadouci += pow(2,i);
    }
  }
  // odebereme sirky do doucho potu, konstrukce zaruuje, e to bude vdy kladn
  k = hromadka[h] - zadouci;
}



int main() {
  char c;
  int i;

  srand(time(NULL));
  // program neprovd dnou kontrolu vstup od uivatele  
  cout << "S kolika hromadkami budeme hrat?"<<endl;
  cin >> n;
  hromadka.resize(n);    

  cout << "Jake jsou pocty sirek na hromadkach? Zadej seznam " << n << " cisel." <<endl;
  for (i=0; i<n; i++)
    cin >> hromadka[i];
  
  cout << "Chces zacinat? (A/N)" <<endl;
  cin >> c;
  hrac_na_tahu = (c == 'A');  
  cout <<endl;
  
  while (! vsechny_hromadky_prazdne()) {
    cout << endl << "Aktualni pocty sirek:";
    for (i=0; i<n; i++) cout << " "<< hromadka[i];
    cout << endl;

    int h, k;
    if (hrac_na_tahu) {
      cout << "Z ktere hromady beres? " ;
      cin >> h;
      h = h - 1;
      cout << "Kolik sirek? (nejvyse " << hromadka[h] << ")  " ;
      cin >> k;
    } else {
      optimalni_tah(h, k);
      // pokud vtzn tah neexistuje, hrajeme nhodn
      if (k == 0) 
	nahodny_tah(h, k);
      cout << "Muj tah: hromadka: "<< h+1<<", sirek: "<< k << endl;
    }
    hromadka[h] -= k;
    hrac_na_tahu = ! hrac_na_tahu;
  }
  cout << "Konec hry"<< endl;
  
  return 0;
}
